@@ -33,9 +33,7 @@
 
 #include <glib/gi18n-lib.h>
 
-#include <gio/gioerror.h>
-#include <gio/gthemedicon.h>
-#include <gio/gfileicon.h>
+#include <gio/gio.h>
 #include "gvfsdaemondbus.h"
 #include <gvfsdaemonprotocol.h>
 #include <gdaemonvfs.h>
@@ -46,6 +44,7 @@
 typedef struct {
   int extra_fd;
   int extra_fd_count;
+  char *async_dbus_id;
   
   /* Only used for async connections */
   GHashTable *outstanding_fds;
@@ -66,6 +65,15 @@ static GHashTable *obj_path_map = NULL;
 G_LOCK_DEFINE_STATIC(obj_path_map);
 
 static void setup_async_fd_receive (VfsConnectionData *connection_data);
+static void invalidate_local_connection (const char *dbus_id,
+					 GError **error);
+  
+
+GQuark
+_g_vfs_error_quark (void)
+{
+  return g_quark_from_static_string ("g-vfs-error-quark");
+}
 
 static gpointer
 vfs_dbus_init (gpointer arg)
@@ -127,16 +135,35 @@ vfs_connection_filter (DBusConnection     *connection,
   DBusHandlerResult res;
   DBusHandleMessageFunction callback;
   GObject *data;
+  VfsConnectionData *connection_data;
+  const char *path;
 
   callback = NULL;
   data = NULL;
+
+  if (dbus_message_is_signal (message,
+			      DBUS_INTERFACE_LOCAL,
+			      "Disconnected"))
+    {
+      connection_data = dbus_connection_get_data (connection, vfs_data_slot);
+      if (connection_data->async_dbus_id)
+	{
+	  _g_daemon_vfs_invalidate_dbus_id (connection_data->async_dbus_id);
+	  G_LOCK (async_map);
+	  g_hash_table_remove (async_map, connection_data->async_dbus_id);
+	  G_UNLOCK (async_map);
+	}
+      
+      return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    }
   
+  path = dbus_message_get_path (message);
   G_LOCK (obj_path_map);
-  if (obj_path_map)
+  if (obj_path_map && path)
     {
       entry = g_hash_table_lookup (obj_path_map,
-				   dbus_message_get_path (message));
-
+				   path);
+      
       if (entry)
 	{
 	  callback = entry->callback;
@@ -160,8 +187,9 @@ static void
 connection_data_free (gpointer p)
 {
   VfsConnectionData *data = p;
-  
-  close (data->extra_fd);
+
+  if (data->extra_fd != -1)
+    close (data->extra_fd);
 
   if (data->extra_fd_source)
     {
@@ -171,6 +199,8 @@ connection_data_free (gpointer p)
 
   if (data->outstanding_fds)
     g_hash_table_destroy (data->outstanding_fds);
+
+  g_free (data->async_dbus_id);
   
   g_free (data);
 }
@@ -181,7 +211,7 @@ vfs_connection_setup (DBusConnection *connection,
 		      gboolean async)
 {
   VfsConnectionData *connection_data;
-  
+
   connection_data = g_new0 (VfsConnectionData, 1);
   connection_data->extra_fd = extra_fd;
   connection_data->extra_fd_count = 0;
@@ -223,6 +253,16 @@ async_connection_accept_new_fd (VfsConnectionData *data,
   int new_fd;
   int fd_id;
   OutstandingFD *outstanding_fd;
+
+  if (condition & G_IO_HUP)
+    {
+      close (data->extra_fd);
+      data->extra_fd = -1;
+      g_source_destroy (data->extra_fd_source);
+      g_source_unref (data->extra_fd_source);
+      data->extra_fd_source = NULL;
+      return;
+    }
   
   fd_id = data->extra_fd_count;
   new_fd = _g_socket_receive_fd (data->extra_fd);
@@ -261,7 +301,7 @@ setup_async_fd_receive (VfsConnectionData *connection_data)
   
   
   connection_data->extra_fd_source =
-    __g_fd_source_new (connection_data->extra_fd, POLLIN, NULL);
+    __g_fd_source_new (connection_data->extra_fd, POLLIN|POLLERR, NULL);
   g_source_set_callback (connection_data->extra_fd_source,
 			 (GSourceFunc)async_connection_accept_new_fd,
 			 connection_data, NULL);
@@ -357,7 +397,12 @@ close_and_unref_connection (void *data)
 static void
 set_connection_for_async (DBusConnection *connection, const char *dbus_id)
 {
+  VfsConnectionData *data;
+  
   G_LOCK (async_map);
+  data = dbus_connection_get_data (connection, vfs_data_slot);
+  data->async_dbus_id = g_strdup (dbus_id);
+
   if (async_map == NULL)
     async_map = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, close_and_unref_connection);
       
@@ -654,9 +699,6 @@ _g_vfs_daemon_call_sync (DBusMessage *message,
   if (connection == NULL)
     return NULL;
 
-  /* TODO: Handle errors below due to unmount and invalidate the
-     sync connection cache */
-  
   if (g_cancellable_set_error_if_cancelled (cancellable, error))
     return NULL;
 
@@ -679,11 +721,12 @@ _g_vfs_daemon_call_sync (DBusMessage *message,
 					    G_VFS_DBUS_TIMEOUT_MSECS))
 	_g_dbus_oom ();
       
-      if (pending == NULL)
+      if (pending == NULL ||
+	  !dbus_connection_get_is_connected (connection))
 	{
-	  g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		       "Error while getting peer-to-peer dbus connection: %s",
-		       "Connection is closed");
+	  if (pending)
+	    dbus_pending_call_unref (pending);
+	  invalidate_local_connection (dbus_id, error);
 	  goto out;
 	}
 
@@ -768,7 +811,17 @@ _g_vfs_daemon_call_sync (DBusMessage *message,
 							 &derror);
       if (!reply)
 	{
-	  _g_error_from_dbus (&derror, error);
+	  if (dbus_error_has_name (&derror, DBUS_ERROR_NO_REPLY) &&
+	      !dbus_connection_get_is_connected (connection))
+	    {
+	      /* The mount for this connection died, we invalidate
+	       * the caches, and then caller needs to retry.
+	       */
+
+	      invalidate_local_connection (dbus_id, error);
+	    }
+	  else
+	    _g_error_from_dbus (&derror, error);
 	  dbus_error_free (&derror);
 	  goto out;
 	}
@@ -816,6 +869,24 @@ free_local_connections (ThreadLocalConnections *local)
   g_free (local);
 }
 
+static void
+invalidate_local_connection (const char *dbus_id,
+			     GError **error)
+{
+  ThreadLocalConnections *local;
+  
+  _g_daemon_vfs_invalidate_dbus_id (dbus_id);
+
+  local = g_static_private_get (&local_connections);
+  if (local)
+    g_hash_table_remove (local->connections, dbus_id);
+  
+  g_set_error_literal (error,
+		       G_VFS_ERROR,
+		       G_VFS_ERROR_RETRY,
+		       "Cache invalid, retry (internally handled)");
+}
+
 DBusConnection *
 _g_dbus_connection_get_sync (const char *dbus_id,
 			     GError **error)
@@ -841,22 +912,49 @@ _g_dbus_connection_get_sync (const char *dbus_id,
     }
 
   if (dbus_id == NULL)
-    connection = local->session_bus;
+    {
+      /* Session bus */
+      
+      if (local->session_bus)
+	{
+	  if (dbus_connection_get_is_connected (local->session_bus))
+	    return local->session_bus;
+
+	  /* Session bus was disconnected, re-connect */
+	  local->session_bus = NULL;
+	  dbus_connection_unref (local->session_bus);
+	}
+    }
   else
-    connection = g_hash_table_lookup (local->connections, dbus_id);
-  
-  if (connection != NULL)
-    return connection;
+    {
+      /* Mount daemon connection */
+      
+      connection = g_hash_table_lookup (local->connections, dbus_id);
+      if (connection != NULL)
+	{
+	  if (!dbus_connection_get_is_connected (connection))
+	    {
+	      /* The mount for this connection died, we invalidate
+	       * the caches, and then caller needs to retry.
+	       */
+
+	      invalidate_local_connection (dbus_id, error);
+	      return NULL;
+	    }
+	  
+	  return connection;
+	}
+    }
 
   dbus_error_init (&derror);
-  
+
   if (local->session_bus == NULL)
     {
       bus = dbus_bus_get_private (DBUS_BUS_SESSION, &derror);
       if (bus == NULL)
 	{
 	  g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		       "Couldn't get main dbus connection: %s\n",
+		       "Couldn't get main dbus connection: %s",
 		       derror.message);
 	  dbus_error_free (&derror);
 	  return NULL;
@@ -865,7 +963,7 @@ _g_dbus_connection_get_sync (const char *dbus_id,
       local->session_bus = bus;
 
       if (dbus_id == NULL)
-	return bus;
+	return bus; /* We actually wanted the session bus, so done */
     }
   
   message = dbus_message_new_method_call (dbus_id,
