@@ -33,7 +33,7 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 
-#include <libsoup/soup.h>
+#include <libsoup/soup-gnome.h>
 #include "gvfsbackendhttp.h"
 #include "gvfsjobopenforread.h"
 #include "gvfsjobread.h"
@@ -79,7 +79,8 @@ g_vfs_backend_http_finalize (GObject *object)
 static void
 g_vfs_backend_http_init (GVfsBackendHttp *backend)
 {
-   const char         *debug;
+  const char         *debug;
+  SoupSessionFeature *proxy_resolver;
 
   g_vfs_backend_set_user_visible (G_VFS_BACKEND (backend), FALSE);  
 
@@ -91,6 +92,12 @@ g_vfs_backend_http_init (GVfsBackendHttp *backend)
                                                                 "gvfs/" VERSION,
                                                                 NULL);
 
+  /* Proxy handling */
+  proxy_resolver = g_object_new (SOUP_TYPE_PROXY_RESOLVER_GNOME, NULL);
+  soup_session_add_feature (backend->session, proxy_resolver);
+  soup_session_add_feature (backend->session_async, proxy_resolver);
+  g_object_unref (proxy_resolver);
+
   /* Logging */
   debug = g_getenv ("GVFS_HTTP_DEBUG");
   if (debug)
@@ -107,8 +114,8 @@ g_vfs_backend_http_init (GVfsBackendHttp *backend)
         level = SOUP_LOGGER_LOG_MINIMAL;
 
       logger = soup_logger_new (level, DEBUG_MAX_BODY_SIZE);
-      soup_logger_attach (logger, backend->session);
-      soup_logger_attach (logger, backend->session_async);
+      soup_session_add_feature (backend->session, SOUP_SESSION_FEATURE (logger));
+      soup_session_add_feature (backend->session_async, SOUP_SESSION_FEATURE (logger));
       g_object_unref (logger);
     }
 
@@ -346,6 +353,7 @@ open_for_read_ready (GObject      *source_object,
                                 error->message);
 
       g_error_free (error);
+      g_object_unref (stream);
       return;
     }
 
@@ -371,6 +379,8 @@ try_open_for_read (GVfsBackend        *backend,
   msg = soup_message_new_from_uri (SOUP_METHOD_GET, uri);
   soup_uri_free (uri);
 
+  soup_message_body_set_accumulate (msg->response_body, FALSE);
+
   stream = soup_input_stream_new (op_backend->session_async, msg);
   g_object_unref (msg);
 
@@ -607,9 +617,16 @@ query_info_ready (SoupSession *session,
                                    "Last-Modified");
   if (text)
     {
+      SoupDate *sd;
       GTimeVal tv;
-      if (g_time_val_from_iso8601 (text, &tv))
-        g_file_info_set_modification_time (info, &tv);
+
+      sd = soup_date_new_from_string(text);
+      if (sd)
+        {
+          soup_date_to_timeval (sd, &tv);
+	  g_file_info_set_modification_time (info, &tv);
+          soup_date_free (sd);
+        }
     }
 
 
