@@ -36,6 +36,8 @@ struct OPAQUE_TYPE__GVfsBackendTrash
 
   TrashWatcher *watcher;
   TrashRoot *root;
+
+  guint thaw_timeout_id;
 };
 
 G_DEFINE_TYPE (GVfsBackendTrash, g_vfs_backend_trash, G_VFS_TYPE_BACKEND);
@@ -330,6 +332,28 @@ trash_backend_close_read (GVfsBackend       *backend,
   return TRUE;
 }
 
+static gboolean
+trash_backend_thaw_callback (gpointer user_data)
+{
+  GVfsBackendTrash *backend = user_data;
+
+  trash_root_thaw (backend->root);
+
+  backend->thaw_timeout_id = 0;
+  return FALSE;
+}
+
+static void
+trash_backend_schedule_thaw (GVfsBackendTrash *backend)
+{
+  if (backend->thaw_timeout_id)
+    g_source_remove (backend->thaw_timeout_id);
+
+  backend->thaw_timeout_id = g_timeout_add (200,
+                                            trash_backend_thaw_callback,
+                                            backend);
+}
+
 static gboolean
 trash_backend_delete (GVfsBackend   *vfs_backend,
                       GVfsJobDelete *job,
@@ -364,6 +388,7 @@ trash_backend_delete (GVfsBackend   *vfs_backend,
             {
               if (trash_item_delete (item, &error))
                 {
+                  trash_backend_schedule_thaw (backend);
                   g_vfs_job_succeeded (G_VFS_JOB (job));
                   trash_item_unref (item);
 
@@ -454,21 +479,36 @@ trash_backend_add_info (TrashItem *item,
                         GFileInfo *info,
                         gboolean   is_toplevel)
 {
-  GFile *original;
-
-  if (is_toplevel && item)
+  if (is_toplevel)
     {
+      const gchar *delete_date;
+      GFile *original;
+
+      g_assert (item != NULL);
+
       original = trash_item_get_original (item);
 
       if (original)
         {
-          gchar *basename;
+          gchar *basename, *path;
+
+          path = g_file_get_path (original);
+          basename = g_filename_display_basename (path);
 
-          basename = g_file_get_basename (original);
-          /* XXX: utf8ify or something... */
           g_file_info_set_display_name (info, basename);
+          g_file_info_set_attribute_byte_string (info,
+                                                 "trash::orig-path",
+                                                 path);
           g_free (basename);
+          g_free (path);
         }
+
+      delete_date = trash_item_get_delete_date (item);
+
+      if (delete_date)
+        g_file_info_set_attribute_string (info,
+                                          "trash::deletion-date",
+                                          delete_date);
     }
 
   g_file_info_set_attribute_boolean (info,
